using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Text;

namespace GDALWrapper.OGR
{
	public enum OGRwkbGeometryType :uint
	{
		wkbUnknown = 0,							// non-standard
		wkbPoint = 1,							// rest are standard WKB type codes
		wkbLineString = 2,
		wkbPolygon = 3,
		wkbMultiPoint = 4,
		wkbMultiLineString = 5,
		wkbMultiPolygon = 6,
		wkbGeometryCollection = 7,
		wkbNone = 100,							// non-standard, for pure attribute records
		wkbLinearRing = 101,					// non-standard, just for createGeometry()
		wkbPoint25D = (uint)0x80000001,			// 2.5D extensions as per 99-402
		wkbLineString25D = (uint)0x80000002,
		wkbPolygon25D = (uint)0x80000003,
		wkbMultiPoint25D = (uint)0x80000004,
		wkbMultiLineString25D = (uint)0x80000005,
		wkbMultiPolygon25D = (uint)0x80000006,
		wkbGeometryCollection25D = (uint)0x80000007
	};

	/// <summary>
	/// The base geometry class. Since the OGR wrapper classes only store handle references to 
	/// the unmanaged objects, this abstract class is not abstract in the wrapper, to enable
	/// instantiation of any base geometry types. This is necessary in a case such as determining
	/// the geometry type from a geometry list.
	/// </summary>
	public class OgrGeometry
	{
		internal HandleRef geometryHandle;

		public const uint wkbFlattenMask = 0x80000000;

		#region public properties

		/// <summary>
		/// The number of points in the geometry.
		/// </summary>
		public int NumberPoints
		{
			get
			{
				return OgrInterop.OGR_G_GetPointCount(this.geometryHandle);
			}
		}

		#endregion

		#region constructors

		public OgrGeometry(IntPtr ptrGeometry)
		{
			this.geometryHandle = new HandleRef(this, ptrGeometry);
		}

		public OgrGeometry(OGRwkbGeometryType geometryType)
		{
			this.geometryHandle = new HandleRef(this,
				OgrInterop.OGR_G_CreateGeometry(geometryType));
		}

		public OgrGeometry(IntPtr ptrGeometry, OGRwkbGeometryType geometryType)
		{
			this.geometryHandle = new HandleRef(this, ptrGeometry);
		}

		#endregion

		#region public methods

		/// <summary>
		/// Fetch geometry type.
		/// </summary>
		/// <param name="flatten">Flattens any 2.5D geometry types to 2D.</param>
		/// <returns>Geometry type</returns>
		public OGRwkbGeometryType GetGeometryType(bool flatten)
		{
			OGRwkbGeometryType geoType = (OGRwkbGeometryType)(OgrInterop.OGR_G_GetGeometryType(this.geometryHandle));

			if (flatten && (((uint)geoType & OgrGeometry.wkbFlattenMask) > 0))
			{
				// flatten any 2.5D types
				geoType = (OGRwkbGeometryType)((uint)geoType - OgrGeometry.wkbFlattenMask);
			}
			return geoType;
		}

		/// <summary>
		/// Fetch a point in line string or a point geometry.
		/// </summary>
		/// <param name="pointIndex">The vertex to fetch, from 0 to NumPoints-1, zero for a point.</param>
		/// <param name="x">Value of x coordinate</param>
		/// <param name="y">Value of y coordinate</param>
		/// <param name="z">Value of z coordinate</param>
		public void GetPoint(int pointIndex, out double x, out double y, out double z)
		{
			OgrInterop.OGR_G_GetPoint(this.geometryHandle, pointIndex, out x, out y, out z);
		}

		#endregion
	}
}
