using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Text;

namespace GDALWrapper.OGR
{
	public class OgrDriver
	{
		internal HandleRef driverHandle;

		#region public properties

		public string Name
		{
			get
			{
				return OgrInterop.OGR_Dr_GetName(this.driverHandle);
			}
		}

		#endregion

		#region constructors

		/// <summary>
		/// Creates a new OgrDriver object. Does not initiate the driver handle,
		/// so the OpenDataSource() method must be called prior to using.
		/// </summary>
		public OgrDriver()
		{
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="driverIndex">The driver index, from 0 to (GetDriverCount - 1).</param>
		public OgrDriver(int driverIndex)
        {
			driverHandle = new HandleRef(this, OgrInterop.OGRGetDriver(driverIndex));
		}

		#endregion

		#region public methods

		/// <summary>
		/// This function attempts to create a new data source based on the passed driver.
		/// </summary>
		/// <param name="folderName">
		/// The folder in which the datasource output will be created.
		/// </param>
		/// <param name="options">
		/// The options argument can be used to control driver specific creation options.
		/// These options are normally documented in the format specific documentation.
		/// </param>
		/// <returns>OgrDataSource object if successful. Null otherwise.</returns>
		public OgrDataSource CreateDataSource(string folderName, List<string> options)
		{
			string[] optionsArray = options.ToArray();

			IntPtr ptrDS = OgrInterop.OGR_Dr_CreateDataSource(this.driverHandle,
				folderName, ref optionsArray);

			if (ptrDS == IntPtr.Zero)
			{
				return null;
			}
			return new OgrDataSource(ptrDS);
		}

		/// <summary>
		/// Open a file / data source with one of the registered drivers.
		/// </summary>
		/// <param name="folderName">The name of the file, or data source to open. </param>
		/// <param name="accessType">Read-only or read-write access.</param>
		/// <returns>OgrDataSource object if successful.</returns>
		/// <exception cref="OgrException">Thrown if the datasource could not be opened.</exception>
		public OgrDataSource OpenDataSource(string folderName, OpenAccessTypes accessType)
		{
			OgrDataSource dataSource = new OgrDataSource(
				OgrInterop.OGROpen(folderName, (int)accessType, this.driverHandle));

			if (dataSource.dataSourceHandle.Handle == IntPtr.Zero)
			{
				string message = GDALWrapper.Gdal.GdalInterOp.CPLGetLastErrorMsg();
				throw new OgrException(message);
			}

			return dataSource;
		}

		/// <summary>
		/// Deletes a datasource.
		/// </summary>
		/// <param name="dataSource">The OgrDataSource to delete.</param>
		/// <returns>Error code.</returns>
		public OGRErr DeleteDataSource(OgrDataSource dataSource)
		{
			return dataSource.Release();
		}

		#endregion

		#region public static methods

		/// <summary>
		/// Gets a driver by name. e.g. "MapInfo File"
		/// </summary>
		/// <param name="driverName">The driver name.</param>
		/// <returns>OgrDriver object if successful. Null otherwise.</returns>
		public static OgrDriver GetDriverByName(string driverName)
		{
			OgrDriver rv = null;

			// find driver by name
			int numDrivers = OgrInterop.OGRGetDriverCount();
			for (int i = 0; i < numDrivers; i++)
			{
				OgrDriver driver = new OgrDriver(i);
				if (String.Compare(driver.Name, driverName, true) == 0)
				{
					rv = driver;
					break;
				}
			}
			return rv;
		}

		/// <summary>
		/// Registers all Ogr drivers.
		/// </summary>
		public static void OgrRegisterAll()
		{
			OgrInterop.OGRRegisterAll();
		}

		#endregion
	}
}
