using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Text;

namespace GDALWrapper.OGR
{
	public enum OGRFieldType
	{
		OFTInteger = 0,			// Simple 32bit integer                   
		OFTIntegerList = 1,		// List of 32bit integers
		OFTReal = 2,			// Double Precision floating point
		OFTRealList = 3,		// List of doubles
		OFTString = 4,			// String of ASCII chars
		OFTStringList = 5,		// Array of strings
		OFTWideString = 6,		// Double byte string (unsupported)
		OFTWideStringList = 7,	// List of wide strings (unsupported
		OFTBinary = 8			// Raw Binary data (unsupported
	};

	public class OgrFieldDefinition
	{
		internal HandleRef fieldDefnHandle;

		#region constructors

		public OgrFieldDefinition(IntPtr ptrFieldDefinition)
		{
			this.fieldDefnHandle = new HandleRef(this, ptrFieldDefinition);
		}

		public OgrFieldDefinition(string fieldDefinitionName, OGRFieldType fieldType)
		{
			IntPtr ptrFieldDefn = OgrInterop.OGR_Fld_Create(fieldDefinitionName, fieldType);
			if (ptrFieldDefn != IntPtr.Zero)
			{
				this.fieldDefnHandle = new HandleRef(this, ptrFieldDefn);
			}
			else
			{
				throw new OgrException("Could not create field definition");
			}
		}

		#endregion

		#region public properties

		/// <summary>
		/// Gets or sets the formatting width for this field.
		/// </summary>
		public int Width
		{
			get
			{
				return OgrInterop.OGR_Fld_GetWidth(this.fieldDefnHandle);
			}
			set
			{
				OgrInterop.OGR_Fld_SetWidth(this.fieldDefnHandle, value);
			}
		}

		/// <summary>
		/// Gets or sets the formatting precision for this field.
		/// This should normally be zero for fields of types other than OFTReal.
		/// </summary>
		public int Precision
		{
			get
			{
				return OgrInterop.OGR_Fld_GetPrecision(this.fieldDefnHandle);
			}
			set
			{
				OgrInterop.OGR_Fld_SetPrecision(this.fieldDefnHandle, value);
			}
		}

		/// <summary>
		/// Gets the OGRFieldType of this field.
		/// </summary>
		public OGRFieldType FieldType
		{
			get
			{
				return OgrInterop.OGR_Fld_GetType(this.fieldDefnHandle);
			}
		}

		/// <summary>
		/// Gets the name of this field.
		/// </summary>
		public string FieldName
		{
			get
			{
				return Marshal.PtrToStringAnsi(OgrInterop.OGR_Fld_GetNameRef(this.fieldDefnHandle));
			}
		}

		#endregion

		#region public static methods

		/// <summary>
		/// Gets the OGRFieldType from a .NET object type.
		/// </summary>
		/// <param name="objectType">The type of the field.</param>
		/// <returns>OGRFieldType</returns>
		/// <exception cref="NotSupportedException">If the objectType is not supported.</exception>
		public static OGRFieldType GetFieldTypeFromType(Type objectType)
		{
			OGRFieldType fieldType;

			if (objectType == typeof(int) || objectType == typeof(long))
			{
				fieldType = OGRFieldType.OFTInteger;
			}
			else if (objectType == typeof(double))
			{
				fieldType = OGRFieldType.OFTReal;
			}
			else
			{
				throw new NotSupportedException(objectType.ToString() + " not supported.");
			}

			return fieldType;
		}

		#endregion
	}
}
