using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Text;

namespace GDALWrapper.OGR
{
	public class OgrLayer
	{
		internal HandleRef layerHandle;

		#region constructor

		public OgrLayer(IntPtr ptrLayer)
		{
			this.layerHandle = new HandleRef(this, ptrLayer);
		}

		#endregion

		#region public methods

		/// <summary>
		/// Create a new field on a layer. You must use this to create new fields on a real layer.
		/// Internally the OGRFeatureDefn for the layer will be updated to reflect the new field.
		/// Applications should never modify the OGRFeatureDefn used by a layer directly.
		/// </summary>
		/// <param name="fieldDefn">The field definition for the field to be created.</param>
		/// <param name="approxOK">
		/// If TRUE, the field may be created in a slightly different form depending on the
		/// limitations of the format driver.
		/// </param>
		/// <returns>Error code</returns>
		public OGRErr CreateField(OgrFieldDefinition fieldDefn, bool approxOK)
		{
			return OgrInterop.OGR_L_CreateField(this.layerHandle,
				fieldDefn.fieldDefnHandle, (approxOK ? 1 : 0));
		}

		/// <summary>
		/// Called CreateFeature in the C API, but is really adding a feature to 
		/// the layer.
		/// </summary>
		/// <param name="feature">The feature to be added to this layer.</param>
		public void AddFeature(OgrFeature feature)
		{
			OGRErr error = OgrInterop.OGR_L_CreateFeature(this.layerHandle,
				feature.featureHandle);

			if (error != OGRErr.None)
			{
				string message = "Could not add feature to layer. " + error.ToString();
				throw new OgrException(message);
			}
		}

		/// <summary>
		/// Reset feature reading to start on the first feature. This affects GetNextFeature().
		/// </summary>
		public void ResetReading()
		{
			OgrInterop.OGR_L_ResetReading(this.layerHandle);
		}

		/// <summary>
		/// Fetch the next available feature from this layer.
		/// The returned feature becomes the responsiblity of the caller to delete.
		/// 
		/// It is critical that all features associated with an OGRLayer
		/// (more specifically an OGRFeatureDefn) be deleted before that layer/datasource is deleted.
		/// Only features matching the current spatial filter (set with SetSpatialFilter()) will be returned.
		/// This function implements sequential access to the features of a layer.
		/// The OGR_L_ResetReading() function can be used to start at the beginning again.
		/// </summary>
		/// <returns>OgrFeature object. Null if the end of the layer has been reached.</returns>
		public OgrFeature GetNextFeature()
		{
			IntPtr ptrFeature = OgrInterop.OGR_L_GetNextFeature(this.layerHandle);
			if (ptrFeature != IntPtr.Zero)
			{
				return new OgrFeature(ptrFeature);
			}
			return null;
		}

		/// <summary>
		/// Fetch the schema information for this layer.
		/// </summary>
		/// <returns>OgrFeatureDefinition object if successful. Null otherwise.</returns>
		public OgrFeatureDefinition GetLayerFeatureDefinition()
		{
			IntPtr ptrFD = OgrInterop.OGR_L_GetLayerDefn(this.layerHandle);
			if (ptrFD != IntPtr.Zero)
			{
				return new OgrFeatureDefinition(ptrFD);
			}
			return null;
		}

		#endregion
	}
}
